/*
 * iwstumbler - part of the iwdrive project
 * author: raymond medeiros (rrm@subterrain.net)
 * requires: libiw2 - wireless-tools pacakge for linux
 */

#include "iwlib.h"

#define MAX_BUFF 128

char *ap[4096];
int count = 0;


void
usage (char *name)
{
  printf ("%s interface delay\n", name);
  exit (-1);
}

int
check_list (struct wireless_info *info)
{

  char *buffer;
  int i;

  buffer = (char *) malloc (MAX_BUFF);
  iw_pr_ether (buffer, info->ap_addr.sa_data);
  for (i = 0; i < count; i++)
    {
      if (strcmp (buffer, ap[i]) == 0)
	{
	  return -1;
	}
    }
// add ap to list
  ap[count] = (char *) malloc (MAX_BUFF);
  strncpy (ap[count], buffer, MAX_BUFF);
  count++;

  return 1;
}


int
setup_socket ()
{
  int skfd = -1;
  if ((skfd = iw_sockets_open ()) < 0)
    {
      perror ("socket");
      exit (-1);
    }
  return skfd;
}

int
display_info (struct wireless_info *info)
{
  char buffer[MAX_BUFF];
  // printf ("::FOUND::\n");
  printf ("ssid: %s, nid: %X, mode: %s\n", info->essid,
	  info->nwid.value, iw_operation_mode[info->mode]);
  if (info->has_freq)
    {
      if (info->freq < KILO)
	printf ("chan: %g, ", info->freq);
      else
	{
	  if (info->freq >= GIGA)
	    printf ("freq: %gGHz, ", info->freq / GIGA);
	  else
	    {
	      if (info->freq >= MEGA)
		printf ("freq: %gMHz, ", info->freq / MEGA);
	      else
		printf ("freq: %gkHz, ", info->freq / KILO);
	    }
	}
    }

  printf ("ap: %s, ", iw_pr_ether (buffer, info->ap_addr.sa_data));
  printf ("br: ");
  if (info->bitrate.value >= GIGA)
    printf ("%gGb/s, ", info->bitrate.value / GIGA);
  else if (info->bitrate.value >= MEGA)
    printf ("%gMb/s", info->bitrate.value / MEGA);
  else
    printf ("%gkb/s", info->bitrate.value / KILO);
  iw_print_stats (buffer, &info->stats.qual, &info->range, info->has_range);
  printf ("Link: \n%s\n\n", buffer);
  return 0;
}

int
get_info (int skfd, char *ifname, struct wireless_info *info)
{
  struct iwreq wrq;
  /* Get wireless name */
  strncpy (wrq.ifr_name, ifname, IFNAMSIZ);
  if (ioctl (skfd, SIOCGIWNAME, &wrq) < 0)
    /* If no wireless name : no wireless extensions */
    return (-1);
  else
    strcpy (info->name, wrq.u.name);
  /* Get ranges */
  if (iw_get_range_info (skfd, ifname, &(info->range)) >= 0)
    info->has_range = 1;
  /* Get network ID */
  strncpy (wrq.ifr_name, ifname, IFNAMSIZ);
  if (ioctl (skfd, SIOCGIWNWID, &wrq) >= 0)
    {
      info->has_nwid = 1;
      memcpy (&(info->nwid), &(wrq.u.nwid), sizeof (iwparam));
    }

  /* Get frequency / channel */
  strncpy (wrq.ifr_name, ifname, IFNAMSIZ);
  if (ioctl (skfd, SIOCGIWFREQ, &wrq) >= 0)
    {
      info->has_freq = 1;
      info->freq = iw_freq2float (&(wrq.u.freq));
    }

  /* Get sensitivity */
  strncpy (wrq.ifr_name, ifname, IFNAMSIZ);
  if (ioctl (skfd, SIOCGIWSENS, &wrq) >= 0)
    {
      info->has_sens = 1;
      memcpy (&(info->sens), &(wrq.u.sens), sizeof (iwparam));
    }

/* Get encryption information */
  strncpy (wrq.ifr_name, ifname, IFNAMSIZ);
  wrq.u.data.pointer = (caddr_t) info->key;
  wrq.u.data.length = 0;
  wrq.u.data.flags = 0;
  if (ioctl (skfd, SIOCGIWENCODE, &wrq) >= 0)
    {
      info->has_key = 1;
      info->key_size = wrq.u.data.length;
      info->key_flags = wrq.u.data.flags;
    }

  /* Get ESSID */
  strncpy (wrq.ifr_name, ifname, IFNAMSIZ);
  wrq.u.essid.pointer = (caddr_t) info->essid;
  wrq.u.essid.length = 0;
  wrq.u.essid.flags = 0;
  if (ioctl (skfd, SIOCGIWESSID, &wrq) >= 0)
    {
      info->has_essid = 1;
      info->essid_on = wrq.u.data.flags;
    }

  /* Get AP address */
  strncpy (wrq.ifr_name, ifname, IFNAMSIZ);
  if (ioctl (skfd, SIOCGIWAP, &wrq) >= 0)
    {
      info->has_ap_addr = 1;
      memcpy (&(info->ap_addr), &(wrq.u.ap_addr), sizeof (sockaddr));
    }

  /* Get NickName */
  strncpy (wrq.ifr_name, ifname, IFNAMSIZ);
  wrq.u.essid.pointer = (caddr_t) info->nickname;
  wrq.u.essid.length = 0;
  wrq.u.essid.flags = 0;
  if (ioctl (skfd, SIOCGIWNICKN, &wrq) >= 0)
    if (wrq.u.data.length > 1)
      info->has_nickname = 1;
  /* Get bit rate */
  strncpy (wrq.ifr_name, ifname, IFNAMSIZ);
  if (ioctl (skfd, SIOCGIWRATE, &wrq) >= 0)
    {
      info->has_bitrate = 1;
      memcpy (&(info->bitrate), &(wrq.u.bitrate), sizeof (iwparam));
    }

  /* Get RTS threshold */
  strncpy (wrq.ifr_name, ifname, IFNAMSIZ);
  if (ioctl (skfd, SIOCGIWRTS, &wrq) >= 0)
    {
      info->has_rts = 1;
      memcpy (&(info->rts), &(wrq.u.rts), sizeof (iwparam));
    }

  /* Get fragmentation threshold */
  strncpy (wrq.ifr_name, ifname, IFNAMSIZ);
  if (ioctl (skfd, SIOCGIWFRAG, &wrq) >= 0)
    {
      info->has_frag = 1;
      memcpy (&(info->frag), &(wrq.u.frag), sizeof (iwparam));
    }

  /* Get operation mode */
  strncpy (wrq.ifr_name, ifname, IFNAMSIZ);
  if (ioctl (skfd, SIOCGIWMODE, &wrq) >= 0)
    {
      if ((wrq.u.mode < IW_NUM_OPER_MODE) && (wrq.u.mode >= 0))
	info->has_mode = 1;
      info->mode = wrq.u.mode;
    }

  /* Get Power Management settings */
  strncpy (wrq.ifr_name, ifname, IFNAMSIZ);
  wrq.u.power.flags = 0;
  if (ioctl (skfd, SIOCGIWPOWER, &wrq) >= 0)
    {
      info->has_power = 1;
      memcpy (&(info->power), &(wrq.u.power), sizeof (iwparam));
    }

  return (0);
}

int
int_flap (int skfd, char *ifname)
{
  struct iwreq wrq;
  // Setup device name
  strncpy (wrq.ifr_name, ifname, IFNAMSIZ);
  // flap it
  // off
  wrq.u.essid.flags = 0;
  if (ioctl (skfd, SIOCSIWESSID, &wrq) < 0)
    {
      fprintf (stderr, "SIOCSIWESSID off: %s\n", strerror (errno));
      return (-1);
    }

  // on
  wrq.u.essid.length = 0;
  wrq.u.essid.flags = 1;
  if (ioctl (skfd, SIOCSIWESSID, &wrq) < 0)
    {
      fprintf (stderr, "SIOCSIWESSID on: %s\n", strerror (errno));
      return (-1);
    }
  return 0;
}

int
main (int argc, char **argv)
{
  int skfd, timer;
  struct wireless_info info;
  if (argc < 3)
    {
      usage (argv[0]);
    }

  timer = atoi (argv[2]) + 1;

// default ap address
  ap[count] = (char *) malloc (MAX_BUFF);
  strncpy(ap[count], "44:44:44:44:44:44", MAX_BUFF);
  count++;

  printf ("iwstumbler started...\n");
  skfd = setup_socket ();
  while (1)
    {
      int_flap (skfd, argv[1]);
      get_info (skfd, argv[1], &info);
      if (check_list (&info) == 1)
	{
	  display_info (&info);
	}
      sleep (timer);
    }
  close (skfd);
}
